@@ -2,10 +2,17 @@ import struct
 import sys
 from enum import EnumMeta
 
-from bare.ast import StructType, BarePrimitive, TypeKind, NamedType, ArrayType, MapType, OptionalType, UnionType
+from bare.bare_ast import StructType, BarePrimitive, TypeKind, NamedType, ArrayType, MapType, OptionalType, UnionType
 
 
 def _pack_varint(data, signed=False):
+    """
+    Encode an integer as varint bytes
+    :param data: number to encode
+    :type data: int
+    :param signed: encode as a signed varint with zigzag encoding
+    :return: bytes
+    """
     result = bytes()
     if signed:
         if data < 0:
@@ -20,6 +27,14 @@ def _pack_varint(data, signed=False):
 
 
 def _unpack_varint(data, offset, signed=False):
+    """
+    Decode varint encoded bytes to a regular integer
+    :param data: input buffer to get the bytes from
+    :type data: bytes
+    :param offset: offset into the input buffer
+    :param signed: decode as a signed varint with zigzag encoding
+    :return int
+    """
     i = 0
     shift = 0
     result = 0
@@ -249,45 +264,35 @@ def _unpack_type(ast_node, instance, module, data, offset):
         raise NotImplementedError("{} is not implemented for decoding".format(ast_node))
 
 
-def pack(instance, member=None):
-    if member is None:
-        ast = instance._ast
-        module = sys.modules[instance.__class__.__module__]
-        return _pack_type(ast, instance, module)
-    else:
-        ast = member._ast
-        module = sys.modules[member.__class__.__module__]
-        untagged = _pack_type(ast, member, module)
-        for type in instance._ast.types:
-            if type.type.name == member.__class__.__name__:
-                index = type.value
-                break
-        else:
-            raise ValueError("{} is not a member of {}".format(member.__class__.__name__, instance.__name__))
-        tag = bytes()
-        tag += _pack_primitive(BarePrimitive(TypeKind.UINT), index)
-        return tag + untagged
+def pack(instance, unionclass=None):
+    """
+    Pack an instance of a BARE class into bytes
+    :param instance: object to get the schema and data from
+    :param member: if the instance is a union, this is the object to get the data and sub-schema from
+    :return: bytes
+    """
+    ast = instance._ast
+    module = sys.modules[instance.__class__.__module__]
+
+    if unionclass is not None:
+        return _pack_type(unionclass._ast, instance, module)
+    return _pack_type(ast, instance, module)
 
 
-def unpack(instance, data, union=False, offset=0, primitive=False):
-    if union:
-        module = sys.modules[instance.__module__]
-        tag, offset = _unpack_primitive(BarePrimitive(TypeKind.UINT), data, offset)
-        for type in instance._ast.types:
-            if type.value == tag:
-                break
-        else:
-            raise ValueError("Cannot find type for tag {}".format(tag))
-        result = getattr(module, type.type.name).unpack(data, offset=offset)
-        if isinstance(result, tuple):
-            return result[0]
-        else:
-            return result
-    else:
-        ast = instance._ast
-        module = sys.modules[instance.__class__.__module__]
-        if primitive:
-            value, offset = _unpack_type(ast, instance, module, data, offset)
-            instance.value = value
-            return instance, offset
-        return _unpack_type(ast, instance, module, data, offset)
+def unpack(instance, data, offset=0, primitive=False):
+    """
+    Decode a BARE message into an instance of a class with a BARE schema in it.
+    :param instance: object to load the data into an to get the schema from
+    :param data: the BARE message bytes
+    :param union: BARE message is an union, load the right subclass instead
+    :param offset: offset into the bytes to start encoding
+    :param primitive: bare message doesn't encode a struct, return the raw decoded data instead
+    :return: tuple of the unpacked data, and the number of bytes read from the input
+    """
+    ast = instance._ast
+    module = sys.modules[instance.__class__.__module__]
+    if primitive:
+        value, offset = _unpack_type(ast, instance, module, data, offset)
+        instance.value = value
+        return instance, offset
+    return _unpack_type(ast, instance, module, data, offset)
