@@ -1,6 +1,6 @@
 import argparse
 import bare.parser
-from bare.ast import BareType, BarePrimitive, StructType, TypeKind, BareEnum, UnionType
+from bare.bare_ast import BareType, BarePrimitive, StructType, TypeKind, BareEnum, UnionType
 
 
 def _gen_enum(type):
@@ -48,7 +48,8 @@ def _gen_type(type):
         result += '\t\treturn bare.pack(cls, member)\n\n'
         result += '\t@classmethod\n'
         result += '\tdef unpack(cls, data, offset=0):\n'
-        result += '\t\treturn bare.unpack(cls, data, union=True, offset=offset)\n\n'
+        result += '\t\tinstance = cls()\n'
+        result += '\t\treturn bare.unpack(instance, data, offset=offset)\n\n'
         result += '\n'
         return result
 
@@ -61,7 +62,8 @@ def codegen(schema, output, indent, skip=None):
     result = 'from collections import OrderedDict\n'
     result += 'from enum import Enum\n\n'
     result += 'import bare\n'
-    result += 'from bare.ast import TypeKind, BarePrimitive, StructType, OptionalType, NamedType, ArrayType, MapType, UnionType, UnionValue\n\n\n'
+    result += 'from bare.bare_ast import TypeKind, BarePrimitive, StructType, OptionalType, NamedType, ArrayType, ' \
+              'MapType, UnionType, UnionValue\n\n\n'
 
     schema = bare.parser.parse(schema)
 
@@ -78,7 +80,9 @@ def codegen(schema, output, indent, skip=None):
 
     if indent != '\t':
         result = result.replace('\t', indent)
-    output.write(result)
+    if output is not None:
+        output.write(result)
+    return result
 
 
 def main():
