@@ -104,6 +104,11 @@ def _lex_string(data, line, column):
 
 
 def lex_schema(data):
+    """
+    Lex the input schema string into the TOKENS
+    :param data: schema as string
+    :return: generator that produces LexedToken instances
+    """
     pointer = 0
     line = 1
     column = 0
